
import re
import json
import requests
from datetime import datetime
from urllib.parse import urlparse
from contextlib import ExitStack
from unittest.mock import MagicMock, patch
from odoo import release, fields
from odoo.tests import HttpCase, new_test_user, tagged
from .common import EnterpriseDatabaseCommon


@tagged('post_install', '-at_install')
class TestEduTemplate(HttpCase, EnterpriseDatabaseCommon):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.user_teacher = new_test_user(
            cls.env,
            login="teacher@myschool.example.org",
            groups="base.group_portal",
        )

    def setUp(self):
        super().setUp()

        patch_stack = ExitStack()
        self.addCleanup(patch_stack.close)
        patch_stack.enter_context(
            patch('odoo.addons.openerp_enterprise.models.openerp_enterprise.EDU_PREFIX', new='my_edu_'),
        )

        # patch requests for mocked routes
        _origin_Session_request = requests.Session.request

        def _request(session_self, method, url, *args, **kwargs):
            urlp = urlparse(url)
            if urlp.path == "/saas_master/admin-check" and kwargs.get('params'):
                resp = MagicMock()
                if kwargs['params']['db_name'] == 'my_edu_database':
                    resp.status_code = 200
                    resp.raise_for_status.return_value = None
                else:
                    resp.status_code = 403
                    resp.raise_for_status.side_effect = Exception("403 Forbidden")
                return resp

            if urlp.path == "/saas_worker/extend_edu/my_edu_database" \
                    and urlp.hostname.startswith('my_edu_database.'):
                resp = MagicMock()
                resp.status_code = 200
                resp.raise_for_status.return_value = None
                self.send_ping(self.database.db_uuid, {}, payload_update={
                    'dbname': 'my_edu_database',
                })
                return resp

            return _origin_Session_request(session_self, method, url, *args, **kwargs)

        patch_stack.enter_context(patch.object(requests.Session, 'request', autospec=True, side_effect=_request))

        self.database = self.create_database()
        self.send_ping(self.database.db_uuid, {}, payload_update={
            'dbname': 'my_edu_database',
            'version': "%s.0" % re.sub("saas[~-]", "", str(release.version_info[0])),  # last LTS
            'users': [
                {'login': self.user_teacher.login,
                 'name': self.user_teacher.name,
                 'is_admin': True,
                 'odoo_com_uid': str(self.user_teacher.id)},
            ],
        })

    def test_edu_share(self):
        self.start_tour("/", "edu_template_share", login=self.user_teacher.login)
        self.assertEqual(len(self.database.template_ids), 1)
        self.assertTrue(self.database.template_ids.website_published)

    def test_edu_duplicate_as_teacher_warning(self):
        # force ping with a "static" expiration date in the past
        self.send_ping(self.database.db_uuid, {}, payload_update={
            'dbname': 'my_edu_database',
            'expire_date': '2023-04-01 00:00:00',
        })
        self.assertEqual(self.database.expire_date, datetime(2023, 4, 1))
        template = self.database.templatize({'published_for': 'all'})

        resp = self.url_open('/odoo-enterprise/template/check/my_edu_database')
        self.assertEqual(resp.status_code, 200)
        template_check = json.loads(resp.content)
        self.assertEqual(template_check['template'], True)
        self.assertEqual(
            fields.Datetime.from_string(template_check['create_date']),
            datetime(2021, 4, 1),
            "Create date of the database should be 2 years before expiration",
        )
        self.start_tour(
            template._get_access_url(),
            "edu_template_share_as_teacher_warning",
            login=self.user_teacher.login
        )
